
local _,L = ...
local rematch = Rematch
local tooltip = RematchTooltip

function rematch:ShowTooltip(title,body,anchorPoint,relativeTo,relativePoint,xoff,yoff)
	if rematch:UIJustChanged() or (RematchSettings.HideTooltips and not anchorPoint) then
		return -- don't show tooltip if ui just changed or HideTooltips enabled (with no given anchor)
	end
	if not title then -- grab tooltip from calling widget if one isn't passed
		title = self.tooltipTitle
		body = self.tooltipBody
	end

	if not title then return end -- leave if no tooltip to show

	tooltip.Title:SetText(title)
	tooltip.Body:SetText(body)

	-- adjust width based on width of title and bofy
	local titleWidth = min(tooltip.Title:GetStringWidth(),200)
	local width
	if not body then
		width = titleWidth -- for tooltips with just a title, make width the title width
	else
		width = max(titleWidth,200) -- for tooltips with a body make width the max of title or 180
		-- but both title and body width are less than 180, shrink to max width of title and body
		local bodyWidth=tooltip.Body:GetStringWidth()
		if width>titleWidth and width>bodyWidth then
			width = max(titleWidth,bodyWidth)
		end
	end

	tooltip.Title:SetSize(width,0)
	tooltip.Body:SetSize(width,0)
	tooltip:SetSize(width+16,tooltip.Title:GetStringHeight() + (body and tooltip.Body:GetStringHeight()+2 or 0) + 16)

	if anchorPoint=="cursor" then
		tooltip:SetScript("OnUpdate",tooltip.OnUpdate)
	elseif not anchorPoint or anchorPoint==true then
		rematch:SmartAnchor(tooltip,self)
	else
		tooltip:ClearAllPoints()
		tooltip:SetPoint(anchorPoint,relativeTo,relativePoint,xoff,yoff)
	end

	tooltip:Show()
end

function tooltip:OnUpdate(elapsed)
	local x,y = GetCursorPosition()
	local scale = UIParent:GetEffectiveScale()
	x = x/scale
	y = y/scale
	self:ClearAllPoints()
	if x>UIParent:GetWidth()/2 then -- right half of screen
		self:SetPoint("BOTTOMRIGHT",UIParent,"BOTTOMLEFT",x-2,y+2)
	else -- left half of screen
		self:SetPoint("BOTTOMLEFT",UIParent,"BOTTOMLEFT",x+2,y+2)
	end
end

function rematch:HideTooltip()
	tooltip:Hide()
	tooltip:SetScript("OnUpdate",nil)
	RematchTableTooltip:Hide()
end

-- RematchTableTooltip is a tooltip designed for bigger text that doesn't wrap,
-- especially for in-line textures that want vertically-centered fontstrings.
-- Use RematchTooltip for traditional tooltips with wrapping lines of text.
-- info is an ordered table with each entry as a line: {"title is white","rest of lines","are gold"}
function rematch:ShowTableTooltip(anchorTo,info,height)
	if rematch:UIJustChanged() then return end
	height = height or 16 -- height of each line (14 if none passed)
	local tooltip = RematchTableTooltip
	tooltip.lines = tooltip.lines or {}
	local lines = tooltip.lines
	local maxWidth = 0
	-- erase what was in this tooltip
	for _,line in ipairs(lines) do
		line:SetText("")
		line:Hide()
	end
	-- add a line for each entry in the passed table
	for i=1,#info do
		if not lines[i] then -- create line if it doesn't exist
			lines[i] = tooltip:CreateFontString(nil,"ARTWORK",i==1 and "GameFontHighlight" or "GameFontNormal")
			lines[i]:SetJustifyH("LEFT")
			lines[i]:SetJustifyV("CENTER")
		end
		lines[i]:SetHeight(height)
		lines[i]:SetPoint("TOPLEFT",tooltip,"TOPLEFT",8,-8-(i-1)*height)
		lines[i]:SetText(info[i])
		lines[i]:Show()
		maxWidth = max(maxWidth,lines[i]:GetStringWidth())
	end
	-- resize to accommodate the width/lines
	tooltip:SetWidth(maxWidth+16)
	tooltip:SetHeight(#info*height+16)
	tooltip:Show()
	rematch:SmartAnchor(tooltip,anchorTo)
end
